/*
 * GrammarFromXMLParser.cpp
 *
 *  Created on: Oct 12, 2013
 *      Author: Jan Travnicek
 */

#include "GrammarFromXMLParser.h"

#include "../sax/ParserException.h"
#include "../alphabet/LabeledSymbol.h"

#include "../FromXMLParsers.h"

namespace grammar {

Grammar GrammarFromXMLParser::parse(std::list<sax::Token> &input) const {
	return parse(input, std::set<FEATURES>({FEATURES::GRAMMAR, FEATURES::LEFT_LG, FEATURES::LEFT_RG, FEATURES::RIGHT_LG, FEATURES::RIGHT_RG, FEATURES::LG, FEATURES::CFG, FEATURES::EPSILON_FREE_CFG, FEATURES::CNF, FEATURES::GNF, FEATURES::CSG, FEATURES::NON_CONTRACTING_GRAMMAR, FEATURES::UNRESTRICTED_GRAMMAR}));
}

Grammar GrammarFromXMLParser::parse(std::list<sax::Token>& input, const std::set<FEATURES>& features) const {
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "grammar")) {
		if(!features.count(FEATURES::GRAMMAR)) throw exception::AlibException();
		return Grammar(parseUnknownGrammar(input));
	} else if(isToken(input, sax::Token::TokenType::START_ELEMENT, "UnrestrictedGrammar")) {
		if(!features.count(FEATURES::UNRESTRICTED_GRAMMAR)) throw exception::AlibException();
		return Grammar(parseUnrestrictedGrammar(input));
	} else
		throw sax::ParserException(sax::Token("Grammar / UnrestrictedGrammar", sax::Token::TokenType::START_ELEMENT), input.front());
}

bool GrammarFromXMLParser::first(std::list<sax::Token>& input) const {
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "grammar") || isToken(input, sax::Token::TokenType::START_ELEMENT, "UnrestrictedGrammar")) {
		return true;
	} else {
		return false;
	}
}

UnknownGrammar GrammarFromXMLParser::parseUnknownGrammar(std::list<sax::Token> &input) const {
	UnknownGrammar grammar;

	popToken(input, sax::Token::TokenType::START_ELEMENT, "grammar");

	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		if (isToken(input, sax::Token::TokenType::START_ELEMENT, "nonterminalAlphabet")) {
			grammar.setNonterminalAlphabet(parseNonterminalAlphabet(input));
		} else if (isToken(input, sax::Token::TokenType::START_ELEMENT, "terminalAlphabet")) {
			grammar.setTerminalAlphabet(parseTerminalAlphabet(input));
		} else if (isToken(input, sax::Token::TokenType::START_ELEMENT, "initialSymbol")) {
			grammar.setInitialSymbol(parseInitialSymbol(input));
		} else if (isToken(input, sax::Token::TokenType::START_ELEMENT, "rules")) {
			parseRules(input, grammar);
		} else {
			throw sax::ParserException(sax::Token("grammar", sax::Token::TokenType::END_ELEMENT), input.front());
		}
	}

	popToken(input, sax::Token::TokenType::END_ELEMENT, "grammar");
	return grammar;
}

UnrestrictedGrammar GrammarFromXMLParser::parseUnrestrictedGrammar(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "UnrestrictedGrammar");

	std::set<alphabet::Symbol> nonterminalAlphabet = parseNonterminalAlphabet(input);
	std::set<alphabet::Symbol> terminalAlphabet = parseTerminalAlphabet(input);
	alphabet::Symbol initialSymbol = parseInitialSymbol(input);

	UnrestrictedGrammar grammar(initialSymbol);
	grammar.setNonterminalAlphabet(nonterminalAlphabet);
	grammar.setTerminalAlphabet(terminalAlphabet);

	parseRules<UnrestrictedGrammar>(input, grammar);

	popToken(input, sax::Token::TokenType::END_ELEMENT, "UnrestrictedGrammar");
	return grammar;
}

std::set<alphabet::Symbol> GrammarFromXMLParser::parseNonterminalAlphabet(std::list<sax::Token> &input) const {
	std::set<alphabet::Symbol> inputSymbols;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "nonterminalAlphabet");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		inputSymbols.insert(alib::FromXMLParsers::symbolParser.parse(input));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "nonterminalAlphabet");
	return inputSymbols;
}

std::set<alphabet::Symbol> GrammarFromXMLParser::parseTerminalAlphabet(std::list<sax::Token> &input) const {
	std::set<alphabet::Symbol> inputSymbols;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "terminalAlphabet");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		inputSymbols.insert(alib::FromXMLParsers::symbolParser.parse(input));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "terminalAlphabet");
	return inputSymbols;
}

alphabet::Symbol GrammarFromXMLParser::parseInitialSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "initialSymbol");
	alphabet::Symbol blank(alib::FromXMLParsers::symbolParser.parse(input));
	popToken(input, sax::Token::TokenType::END_ELEMENT, "initialSymbol");
	return blank;
}

template<class T>
void GrammarFromXMLParser::parseRules(std::list<sax::Token> &input, T& grammar) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "rules");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		popToken(input, sax::Token::TokenType::START_ELEMENT, "rule");
		parseRule(input, grammar);
		popToken(input, sax::Token::TokenType::END_ELEMENT, "rule");
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "rules");
}

void GrammarFromXMLParser::parseRule(std::list<sax::Token>& input, UnrestrictedGrammar& grammar) const {
	std::vector<alphabet::Symbol> lhs = parseRuleLHS(input);
	std::vector<alphabet::Symbol> rhs = parseRuleRHS(input);
	grammar.addRule(lhs, rhs);
}

void GrammarFromXMLParser::parseRule(std::list<sax::Token>& input, UnknownGrammar& grammar) const {
	UnknownRule rule;

	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		if (isToken(input, sax::Token::TokenType::START_ELEMENT, "lhs")) {
			rule.setLeftHandSide(parseRuleLHS(input));
		} else if (isToken(input, sax::Token::TokenType::START_ELEMENT, "rhs")) {
			rule.setRightHandSide(parseRuleRHS(input));
		} else {
			throw sax::ParserException(sax::Token("rule", sax::Token::TokenType::END_ELEMENT), input.front());
		}
	}

	grammar.addRule(rule);
}

std::vector<alphabet::Symbol> GrammarFromXMLParser::parseRuleLHS(std::list<sax::Token>& input) const {
	std::vector<alphabet::Symbol> lhs;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "lhs");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		lhs.push_back(alib::FromXMLParsers::symbolParser.parse(input));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "lhs");
	return lhs;
}

std::vector<alphabet::Symbol> GrammarFromXMLParser::parseRuleRHS(std::list<sax::Token>& input) const {
	std::vector<alphabet::Symbol> rhs;
	popToken(input, sax::Token::TokenType::START_ELEMENT, "rhs");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		rhs.push_back(alib::FromXMLParsers::symbolParser.parse(input));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "rhs");
	return rhs;
}

} /* namespace grammar */

